@@ -16,6 +16,11 @@
 #include "totem-pl-parser-mini.h"
 #include "totem-pl-parser-private.h"
 
+typedef struct {
+	const char *field;
+	char *ret;
+} ParserResult;
+
 static GMainLoop *loop = NULL;
 static gboolean option_no_recurse = FALSE;
 static gboolean option_debug = FALSE;
@@ -86,6 +91,26 @@ test_resolution_real (const char *base_uri,
 static void
 test_resolution (void)
 {
+	const char * const *schemes;
+	gboolean http_supported = FALSE;
+	GVfs *vfs;
+	guint i;
+
+	vfs = g_vfs_get_default ();
+	if (vfs == NULL)
+		g_error ("gvfs with http support is required to test link resolution");
+	schemes = g_vfs_get_supported_uri_schemes (vfs);
+	if (schemes == NULL)
+		g_error ("gvfs with http support is required to test link resolution");
+	for (i = 0; schemes[i] != NULL; i++) {
+		if (g_str_equal (schemes[i], "http")) {
+			http_supported = TRUE;
+			break;
+		}
+	}
+	if (http_supported == FALSE)
+		g_error ("gvfs is installed but does not support http");
+
 	/* http://bugzilla.gnome.org/show_bug.cgi?id=555417 */
 	g_assert_cmpstr (test_resolution_real ("http://www.yle.fi/player/player.jsp", "288629.asx?s=1000"), ==, "http://www.yle.fi/player/288629.asx?s=1000");
 	g_assert_cmpstr (test_resolution_real ("http://www.yle.fi/player/player.jsp?actionpage=3&id=288629&locale", "288629.asx?s=1000"), ==, "http://www.yle.fi/player/288629.asx?s=1000");
@@ -109,6 +134,8 @@ test_duration (void)
 	g_assert_cmpint (totem_pl_parser_parse_duration ("01:00:01.01", verbose), ==, 3601);
 	g_assert_cmpint (totem_pl_parser_parse_duration ("01:00.01", verbose), ==, 60);
 	g_assert_cmpint (totem_pl_parser_parse_duration ("24.59", verbose), ==, 1499);
+	g_assert_cmpint (totem_pl_parser_parse_duration ("02m25s", verbose), ==, 145);
+	g_assert_cmpint (totem_pl_parser_parse_duration ("2m25s", verbose), ==, 145);
 }
 
 static void
@@ -184,6 +211,17 @@ test_data_get_data (const char *uri, guint *len)
 	return buffer;
 }
 
+static void
+test_videosite (void)
+{
+#ifdef HAVE_QUVI
+	char *uri = "http://www.youtube.com/watch?v=oMLCrzy9TEs";
+
+	g_test_message ("Testing data parsing \"%s\"...", uri);
+	g_assert (totem_pl_parser_can_parse_from_uri (uri, TRUE));
+#endif
+}
+
 static void
 test_parsability (void)
 {
@@ -201,6 +239,7 @@ test_parsability (void)
 		{ "http://live.hujjat.org:7860/main", FALSE, TRUE },
 		{ "http://www.comedycentral.com/sitewide/media_player/videoswitcher.jhtml?showid=934&category=/shows/the_daily_show/videos/headlines&sec=videoId%3D36032%3BvideoFeatureId%3D%3BpoppedFrom%3D_shows_the_daily_show_index.jhtml%3BisIE%3Dfalse%3BisPC%3Dtrue%3Bpagename%3Dmedia_player%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bgateway%3Dshows%3Bsection_1%3Dthe_daily_show%3Bsection_2%3Dvideos%3Bsection_3%3Dheadlines%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bera%3D%27%2Bif_nt_era%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bfla%3D%27%2Bif_nt_Flash%2B%27&itemid=36032&clip=com/dailyshow/headlines/10156_headline.wmv&mswmext=.asx", TRUE, TRUE },
 		{ TEST_SRCDIR "HackerMedley", TRUE, FALSE }, /* From https://bugzilla.redhat.com/show_bug.cgi?id=582850 and http://feeds.feedburner.com/HackerMedley */
+		{ "http://faif.us/feeds/cast-mp3", TRUE, TRUE },
 		{ NULL,  FALSE, FALSE }
 	};
 
@@ -246,26 +285,35 @@ static void
 entry_parsed_cb (TotemPlParser *parser,
 		 const char *uri,
 		 GHashTable *metadata,
-		 char **ret)
+		 ParserResult *res)
 {
-	if (*ret == NULL)
-		*ret = g_strdup (uri);
+	if (res->ret == NULL)
+		res->ret = g_strdup (g_hash_table_lookup (metadata, res->field));
 }
 
-static char *
-parser_test_get_parse_result (const char *uri)
+static void
+entry_parsed_num_cb (TotemPlParser *parser,
+		     const char *uri,
+		     GHashTable *metadata,
+		     guint *ret)
+{
+	*ret = *ret + 1;
+}
+
+static guint
+parser_test_get_num_entries (const char *uri)
 {
 	TotemPlParserResult retval;
-	char *ret = NULL;
+	guint ret = 0;
 	TotemPlParser *pl = totem_pl_parser_new ();
 
-	g_object_set (pl, "recurse", !option_no_recurse,
+	g_object_set (pl, "recurse", FALSE,
 			  "debug", option_debug,
 			  "force", option_force,
 			  "disable-unsafe", option_disable_unsafe,
 			  NULL);
 	g_signal_connect (G_OBJECT (pl), "entry-parsed",
-			  G_CALLBACK (entry_parsed_cb), &ret);
+			  G_CALLBACK (entry_parsed_num_cb), &ret);
 
 	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
 	g_test_message ("Got retval %d for uri '%s'", retval, uri);
@@ -337,13 +385,168 @@ simple_parser_test (const char *uri)
 	return retval;
 }
 
+static char *
+parser_test_get_entry_field (const char *uri, const char *field)
+{
+	TotemPlParserResult retval;
+	TotemPlParser *pl = totem_pl_parser_new ();
+	ParserResult res;
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	res.field = field;
+	res.ret = NULL;
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_cb), &res);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return res.ret;
+}
+
+static void
+playlist_started_title_cb (TotemPlParser *parser,
+			   const char *uri,
+			   GHashTable *metadata,
+			   char **ret)
+{
+	*ret = g_strdup (uri);
+}
+
+static char *
+parser_test_get_playlist_uri (const char *uri)
+{
+	TotemPlParserResult retval;
+	char *ret = NULL;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_object_set (pl, "recurse", !option_no_recurse,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "playlist-started",
+			  G_CALLBACK (playlist_started_title_cb), &ret);
+
+	retval = totem_pl_parser_parse_with_base (pl, uri, option_base_uri, FALSE);
+	g_test_message ("Got retval %d for uri '%s'", retval, uri);
+	g_object_unref (pl);
+
+	return ret;
+}
+
+static void
+test_itms_parsing (void)
+{
+	g_assert_cmpstr (parser_test_get_playlist_uri ("itms://itunes.apple.com/gb/podcast/best-of-chris-moyles-enhanced/id142102961?ign-mpt=uo%3D4"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/moylesen/rss.xml");
+	g_assert_cmpstr (parser_test_get_playlist_uri ("http://itunes.apple.com/gb/podcast/radio-1-mini-mix/id268491175?uo=4"), ==, "http://downloads.bbc.co.uk/podcasts/radio1/r1mix/rss.xml");
+}
+
+static void
+test_lastfm_parsing (void)
+{
+	char *uri;
+
+	g_test_bug ("625823");
+
+	uri = get_relative_uri (TEST_SRCDIR "old-lastfm-output.xspf");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI), ==, "http://freedownloads.last.fm/download/188024406/Kondratiev%2BWinter.mp3");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_ID), ==, "d092a");
+	g_free (uri);
+
+	uri = get_relative_uri (TEST_SRCDIR "new-lastfm-output.xspf");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_DOWNLOAD_URI), ==, "http://freedownloads.last.fm/download/402599273/Yellow.mp3");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_ID), ==, "20a82");
+	g_free (uri);
+}
+
+static void
+test_m3u_separator (void)
+{
+	char *uri;
+
+	g_test_bug ("609091");
+
+	uri = get_relative_uri (TEST_SRCDIR "separator.m3u");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_TITLE), ==, "Music Tech Sessions (Friday 22 January 2010 20:00 - 00:00)");
+	g_free (uri);
+}
+
+static void
+test_parsing_xspf_genre (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "playlist.xspf");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_GENRE), ==, "Test Genre");
+	g_free (uri);
+}
+
+static void
+test_smi_starttime (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "big5.smi");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_STARTTIME), ==, "00:04:00");
+	g_free (uri);
+}
+
+static void
+test_m3u_leading_tabs (void)
+{
+	char *uri;
+	/* From http://media.artistserver.com/tracks/23985/21898/1/1/1/O_G_Money_-_Girl_Gotta_girlfriend_Feat._O_G_Money_Snoop_Dogg.m3u */
+	uri = get_relative_uri (TEST_SRCDIR "O_G_Money_Snoop_Dogg.m3u");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_TITLE), ==, "O G Money - Girl Gotta girlfriend Feat. O G Money, Snoop Dogg");
+	g_free (uri);
+}
+
+static void
+test_directory_recurse (void)
+{
+	char *uri, *path;
+
+	uri = get_relative_uri (TEST_SRCDIR "foo");
+	path = g_filename_from_uri (uri, NULL, NULL);
+	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
+		/* The file inside the directory will be ignored */
+		g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_TITLE), ==, NULL);
+		/* But the parsing will succeed */
+		g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	}
+	g_free (path);
+	g_free (uri);
+}
+
+static void
+test_empty_asx (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "empty-asx.asx");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_free (uri);
+}
+
+static void
+test_empty_pls (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "emptyplaylist.pls");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_free (uri);
+}
+
 static void
 test_parsing_rtsp_text_multi (void)
 {
 	char *uri;
 	g_test_bug ("602127");
 	uri = get_relative_uri (TEST_SRCDIR "602127.qtl");
-	g_assert_cmpstr (parser_test_get_parse_result (uri), ==, "rtsp://host.org/video.mp4");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "rtsp://host.org/video.mp4");
 	g_free (uri);
 }
 
@@ -352,7 +555,16 @@ test_parsing_rtsp_text (void)
 {
 	char *uri;
 	uri = get_relative_uri (TEST_SRCDIR "single-line.qtl");
-	g_assert_cmpstr (parser_test_get_parse_result (uri), ==, "rtsp://host.org/video.mp4");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "rtsp://host.org/video.mp4");
+	g_free (uri);
+}
+
+static void
+test_parsing_content_type (void)
+{
+	char *uri;
+	uri = get_relative_uri (TEST_SRCDIR "HackerMedley");
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_CONTENT_TYPE), ==, "audio/mpeg");
 	g_free (uri);
 }
 
@@ -397,6 +609,18 @@ test_parsing_out_of_order_asx (void)
 	g_assert (result != FALSE);
 }
 
+static void
+test_parsing_num_entries (void)
+{
+	char *uri;
+	guint num;
+
+	uri = get_relative_uri (TEST_SRCDIR "missing-items.pls");
+	num = parser_test_get_num_entries (uri);
+	g_free (uri);
+	g_assert (num == 19);
+}
+
 static void
 test_parsing_404_error (void)
 {
@@ -404,6 +628,28 @@ test_parsing_404_error (void)
 	g_assert (simple_parser_test ("http://live.hujjat.org:7860/main") == TOTEM_PL_PARSER_RESULT_UNHANDLED);
 }
 
+static void
+test_parsing_3gpp_not_ignored (void)
+{
+	char *uri;
+
+	uri = get_relative_uri (TEST_SRCDIR "3gpp-file.mp4");
+	g_test_bug ("594359@bugs.debian.org");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_free (uri);
+}
+
+static void
+test_parsing_mp4_is_flv (void)
+{
+	char *uri;
+
+	uri = get_relative_uri (TEST_SRCDIR "really-flv.mp4");
+	g_test_bug ("620039");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_UNHANDLED);
+	g_free (uri);
+}
+
 static void
 test_parsing_xml_head_comments (void)
 {
@@ -457,6 +703,18 @@ test_parsing_not_asx_playlist (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_wma_asf (void)
+{
+	char *uri;
+	g_test_bug ("639958");
+	/* File from https://bugzilla.gnome.org/show_bug.cgi?id=639958#c5 */
+	uri = get_relative_uri (TEST_SRCDIR "WMA9.1_98_quality_48khz_vbr_s.wma");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "mmsh://195.134.224.231/wma/WMA9.1_98_quality_48khz_vbr_s.wma?MSWMExt=.asf");
+	g_free (uri);
+}
+
 static void
 test_parsing_not_really_php (void)
 {
@@ -468,6 +726,84 @@ test_parsing_not_really_php (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_not_really_php_but_html_instead (void)
+{
+	char *uri;
+	g_test_bug ("624341");
+	/* File from http://www.novabrasilfm.com.br/ao-vivo/audio.php */
+	uri = get_relative_uri (TEST_SRCDIR "audio.php");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_IGNORED);
+	g_free (uri);
+}
+
+
+typedef struct {
+	int count;
+	GMainLoop *mainloop;
+	char *uri;
+} AsyncParseData;
+
+static void
+parse_async_ready (GObject *pl, GAsyncResult *result, gpointer userdata)
+{
+	AsyncParseData *data = userdata;
+	TotemPlParserResult retval;
+
+	retval = totem_pl_parser_parse_finish (TOTEM_PL_PARSER (pl), result, NULL);
+	g_test_message ("Got retval %d for uri '%s'", retval, data->uri);
+	g_test_message ("Parsed entry count is %d for uri '%s'", data->count, data->uri);
+
+	g_assert (data->count > 0);
+
+	g_main_loop_quit (data->mainloop);
+	g_object_unref (pl);
+}
+
+static gboolean
+block_main_loop_idle (gpointer data)
+{
+	/* one second should be long enough to parse a local file */
+	g_usleep (G_USEC_PER_SEC);
+	return FALSE;
+}
+
+static void
+test_async_parsing_signal_order (void)
+{
+	AsyncParseData data;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_test_bug ("631727");
+	/* doesn't really matter what test file we use here, we just want something
+	 * with at least one valid entry in it.
+	 */
+	data.uri = get_relative_uri (TEST_SRCDIR "separator.m3u");
+	data.mainloop = g_main_loop_new (NULL, FALSE);
+	data.count = 0;
+
+	g_object_set (pl, "recurse", FALSE,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_num_cb), &data.count);
+
+	/* we need the mainloop to be busy while the signal emission idle sources are
+	 * added, so we just block it for a while.
+	 */
+	g_idle_add_full (G_PRIORITY_HIGH, block_main_loop_idle, NULL, NULL);
+	totem_pl_parser_parse_async (pl, data.uri, FALSE, NULL, parse_async_ready, &data);
+	g_main_loop_run (data.mainloop);
+
+	/* The number of entries in separator.m3u */
+	g_assert(data.count == 1);
+
+	g_free (data.uri);
+	g_main_loop_unref (data.mainloop);
+}
+
 #define MAX_DESCRIPTION_LEN 128
 #define DATE_BUFSIZE 512
 #define PRINT_DATE_FORMAT "%Y-%m-%dT%H:%M:%SZ"
@@ -622,19 +958,36 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/relative", test_relative);
 		g_test_add_func ("/parser/resolution", test_resolution);
 		g_test_add_func ("/parser/parsability", test_parsability);
+		g_test_add_func ("/parser/videosite", test_videosite);
 		g_test_add_func ("/parser/parsing/hadess", test_parsing_hadess);
 		g_test_add_func ("/parser/parsing/nonexistent_files", test_parsing_nonexistent_files);
 		g_test_add_func ("/parser/parsing/broken_asx", test_parsing_broken_asx);
 		g_test_add_func ("/parser/parsing/404_error", test_parsing_404_error);
+		g_test_add_func ("/parser/parsing/3gpp_not_ignored", test_parsing_3gpp_not_ignored);
+		g_test_add_func ("/parser/parsing/mp4_is_flv", test_parsing_mp4_is_flv);
 		g_test_add_func ("/parser/parsing/out_of_order_asx", test_parsing_out_of_order_asx);
 		g_test_add_func ("/parser/parsing/xml_head_comments", test_parsing_xml_head_comments);
 		g_test_add_func ("/parser/parsing/xml_comment_whitespace", test_parsing_xml_comment_whitespace);
 		g_test_add_func ("/parser/parsing/multi_line_rtsptext", test_parsing_rtsp_text_multi);
 		g_test_add_func ("/parser/parsing/single_line_rtsptext", test_parsing_rtsp_text);
+		g_test_add_func ("/parser/parsing/podcast_content_type", test_parsing_content_type);
 		g_test_add_func ("/parser/parsing/live_streaming", test_parsing_live_streaming);
 		g_test_add_func ("/parser/parsing/xml_mixed_cdata", test_parsing_xml_mixed_cdata);
 		g_test_add_func ("/parser/parsing/not_asx_playlist", test_parsing_not_asx_playlist);
 		g_test_add_func ("/parser/parsing/not_really_php", test_parsing_not_really_php);
+		g_test_add_func ("/parser/parsing/not_really_php_but_html_instead", test_parsing_not_really_php_but_html_instead);
+		g_test_add_func ("/parser/parsing/num_items_in_pls", test_parsing_num_entries);
+		g_test_add_func ("/parser/parsing/xspf_genre", test_parsing_xspf_genre);
+		g_test_add_func ("/parser/parsing/itms_link", test_itms_parsing);
+		g_test_add_func ("/parser/parsing/lastfm-attributes", test_lastfm_parsing);
+		g_test_add_func ("/parser/parsing/m3u_separator", test_m3u_separator);
+		g_test_add_func ("/parser/parsing/smi_starttime", test_smi_starttime);
+		g_test_add_func ("/parser/parsing/m3u_leading_tabs", test_m3u_leading_tabs);
+		g_test_add_func ("/parser/parsing/empty-asx.asx", test_empty_asx);
+		g_test_add_func ("/parser/parsing/emptyplaylist.pls", test_empty_pls);
+		g_test_add_func ("/parser/parsing/dir_recurse", test_directory_recurse);
+		g_test_add_func ("/parser/parsing/async_signal_order", test_async_parsing_signal_order);
+		g_test_add_func ("/parser/parsing/wma_asf", test_parsing_wma_asf);
 
 		return g_test_run ();
 	}
